/*
 * Program to kill a machine's performance by one of
 * several ways (chosen randomly).
*/

#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

void cpu(void)
{
        int i;

        fork();
        fork();
        fork();
        for (;;i++) ;
}

void mem(void)
{
        char *buf;
        size_t large = 1 << 26;
        unsigned int a;
        double scale;
        unsigned long snooze = 3000;

        fork();
        fork();
        buf = calloc(large, 1);
        if (buf == 0) {
                perror("calloc failed");
                exit(1);
        }
        for (;;) {
                scale = (double)random() / (double)RAND_MAX;
                a = (int) (scale * large);
                buf[a]++;
                usleep(snooze);
        }
}

#define PATH "/tmp/big"
#define BUF  512*1024

void io(void)
{
        int fd, i;
        size_t large = 1 << 25;
        char buf[BUF];
        off_t offset;

        fork(); fork();
        if ((fd = open(PATH, O_RDWR|O_CREAT|O_FSYNC, 0644)) < 0) {
                perror(PATH);
                exit(1);
        }
        for (i=0; i<BUF; i++) buf[i] = 'A';
        for (;;) {
                offset = (off_t) ((double)random() / (double)RAND_MAX)
		    * large;
                if (lseek(fd, offset, SEEK_SET) < 0) {
                        fprintf(stderr, "lseek on %s failed: %s\n",
                                PATH, strerror(errno));
                        exit(1);
                }
                write(fd, buf, BUF);
        }
}

int main(int argc, char **argv)
{
        time_t t;
        double r;
        int die;

        if (argc >= 2) {
           die = atoi(argv[1]);
        } else {
            time(&t);
            srandom(t);
            r = (double)random() / (double)RAND_MAX;
            die = (int)(r * 3);
        }
        printf("%d\n", die);

        if (die == 0)
                io();
        else if (die == 1)
                cpu();
        else
                mem();
}
